"""IAAF Hungarian (non-combined-events) scoring system

IAAF has published a book of scoring factors, containing about 183,000
individual data points.  It is alleged that the formula herein gives the 
exact same answers.  We'll find out shortly ;-)

score = floor(b * (performance + a)**2  + c), 1)

See https://github.com/GlaivePro/IaafPoints

"""
from math import floor
from athlib.utils import check_event_code

def score(gender, inout, event_code, performance):
    "Return IAAF (Hungarian) points, or raise AssertionError"
    assert inout in ('IND', 'OUT'), "Must specify indoor (IN) or outdoor (OUT)"
    assert gender in 'MFX', "Gender must be M , F or X. X will be looked up in Male points tables!"
    ec = check_event_code(event_code)
    assert ec is not None, "Unrecognised event code %s" % event_code


    key = (gender, inout, event_code)
    tbl = get_lookup_table()
    (a,b,c) = tbl[key]

    # b = performance in m or sec that gets no points.  (Slower would be an input error!)
    #     Points grow quadratically above it
    # a = multiplier to get all events onto a similar scale
    # c = points shift applied at the end

    # to do:  if performance is worse than a, should get zero points.  e.g. 20 second 100m


    return floor(a * (performance + b)**2 + c) 


_table = None
def get_lookup_table():
    global _table
    if _table is None:
        _table = {}
        for (gender, inout, event_code, a, b, c) in FACTORS:
            key = (gender, inout, event_code)
            value = (a,b,c)
            _table[key] = value
    return _table

FACTORS = [  # Gender, indoor/outdoor, event_code, a, b, c 
  ["M","OUT","100",24.63,-17,0  ],
  ["M","OUT","200",5.08,-35.5,0  ],
  ["M","OUT","300",1.83,-57.2,0  ],
  ["M","OUT","400",1.021,-79,0  ],
  ["M","OUT","600",0.367,-131,0  ],
  ["M","OUT","800",0.198,-182,0  ],
  ["M","OUT","1000",0.1074,-240,0  ],
  ["M","OUT","1500",0.04066,-385,0  ],
  ["M","OUT","MILE",0.0351,-415,0  ],
  ["M","OUT","2000",0.02181,-528,0  ],
  ["M","OUT","3000",0.00815,-840,0  ],
  ["M","OUT","2M",0.00703,-904.8,0  ],
  ["M","OUT","5000",0.002778,-1440,0  ],
  ["M","OUT","10000",0.000524,-3150,0  ],
  ["M","OUT","110M",7.66,-25.8,0  ],
  ["M","OUT","400M",0.546,-95.5,0  ],
  ["M","OUT","2000SC",0.01023,-660,0  ],
  ["M","OUT","3000SC",0.004316,-1020,0  ],
  ["M","OUT","4X100",1.236,-69.5,0  ],
  ["M","OUT","4X200",0.29767,-144,0  ],
  ["M","OUT","4X400",0.05026,-334,0  ],
  ["M","OUT","10K",0.00052841,-3150,0  ],
  ["M","OUT","15K",0.0002162,-4868,0  ],
  ["M","OUT","10M",0.0001852,-5250,0  ],
  ["M","OUT","20K",0.0001147,-6629,0  ],
  ["M","OUT","HM",0.000102,-7020,0  ],
  ["M","OUT","25K",0.00006765,-8536,0  ],
  ["M","OUT","30K",0.00004353,-10531,0  ],
  ["M","OUT","MAR",0.0000191,-15600,0  ],
  ["M","OUT","100K",0.00000177,-48600,0  ],
  ["M","OUT","3KW",0.001209,-1650,0  ],
  ["M","OUT","5KW",0.000436,-2760,0  ],
  ["M","OUT","10KW",0.0001118,-5580,0  ],
  ["M","OUT","20KW",0.00002735,-11400,0  ],
  ["M","OUT","30KW",0.00000893,-19110,0  ],
  ["M","OUT","35KW",0.00000576,-23400,0  ],
  ["M","OUT","50KW",0.00000212,-37200,0  ],
  ["M","OUT","HJ",32.29,11.534,-5000  ],
  ["M","OUT","PV",3.042,39.39,-5000  ],
  ["M","OUT","LJ",1.929,48.41,-5000  ],
  ["M","OUT","TJ",0.4611,98.63,-5000  ],
  ["M","OUT","SP",0.042172,687.7,-20000  ],
  ["M","OUT","DT",0.004007,2232.6,-20000  ],
  ["M","OUT","HT",0.0028038,2669.4,-20000  ],
  ["M","OUT","JT",0.0023974,2886.8,-20000  ],
  ["M","OUT","DEC",9.77e-7,71170,-5000  ],
  ["F","OUT","100",9.92,-22,0  ],
  ["F","OUT","200",2.242,-45.5,0  ],
  ["F","OUT","300",0.7,-77,0  ],
  ["F","OUT","400",0.335,-110,0  ],
  ["F","OUT","600",0.1192,-184,0  ],
  ["F","OUT","800",0.0688,-250,0  ],
  ["F","OUT","1000",0.0382,-330,0  ],
  ["F","OUT","1500",0.0134,-540,0  ],
  ["F","OUT","MILE",0.01165,-580,0  ],
  ["F","OUT","2000",0.006766,-750,0  ],
  ["F","OUT","3000",0.002539,-1200,0  ],
  ["F","OUT","2M",0.002157,-1296.3,0  ],
  ["F","OUT","5000",0.000808,-2100,0  ],
  ["F","OUT","10000",0.0001712,-4500,0  ],
  ["F","OUT","100M",3.98,-30,0  ],
  ["F","OUT","400M",0.208567,-130,0  ],
  ["F","OUT","2000SC",0.0045,-880,0  ],
  ["F","OUT","3000SC",0.001323,-1510,0  ],
  ["F","OUT","4X100",0.3895,-98,0  ],
  ["F","OUT","4X200",0.0795,-212,0  ],
  ["F","OUT","4X400",0.01562,-480,0  ],
  ["F","OUT","10K",0.0001742,-4500,0  ],
  ["F","OUT","15K",0.0000732,-6905,0  ],
  ["F","OUT","10M",0.000063,-7438,0  ],
  ["F","OUT","20K",0.0000396,-9357,0  ],
  ["F","OUT","HM",0.0000353,-9900,0  ],
  ["F","OUT","25K",0.0000228,-12144,0  ],
  ["F","OUT","30K",0.00001426,-15123,0  ],
  ["F","OUT","MAR",0.00000595,-22800,0  ],
  ["F","OUT","100K",8.74e-7,-61200,0  ],
  ["F","OUT","3KW",0.000881,-1871,0  ],
  ["F","OUT","5KW",0.0003246,-3140,0  ],
  ["F","OUT","10KW",0.0000779,-6437,0  ],
  ["F","OUT","20KW",0.0000187,-13200,0  ],
  ["F","OUT","30KW",0.0000069,-21545,0  ],
  ["F","OUT","50KW",0.00000196,-39952,0  ],
  ["F","OUT","HJ",39.34,10.574,-5000  ],
  ["F","OUT","PV",3.953,34.83,-5000  ],
  ["F","OUT","LJ",1.966,49.24,-5000  ],
  ["F","OUT","TJ",0.4282,105.53,-5000  ],
  ["F","OUT","SP",0.0462,657.53,-20000  ],
  ["F","OUT","DT",0.0040277,2227.3,-20000  ],
  ["F","OUT","HT",0.0030965,2540,-20000  ],
  ["F","OUT","JT",0.004073,2214.9,-20000  ],
  ["F","OUT","HEP",0.00000158,55990,-5000  ],
  ["M","IND","50",95.8,-9.2,0  ],
  ["M","IND","55",78.9,-10,0  ],
  ["M","IND","60",68.6,-10.7,0  ],
  ["M","IND","200",5.04,-36,0  ],
  ["M","IND","300",1.803,-58,0  ],
  ["M","IND","400",0.981,-80.6,0  ],
  ["M","IND","500",0.565,-106,0  ],
  ["M","IND","600",0.39,-131,0  ],
  ["M","IND","800",0.1974,-184,0  ],
  ["M","IND","1000",0.1139,-240,0  ],
  ["M","IND","1500",0.042,-386,0  ],
  ["M","IND","MILE",0.0369,-415,0  ],
  ["M","IND","2000",0.0226,-528,0  ],
  ["M","IND","3000",0.008322,-840,0  ],
  ["M","IND","2M",0.00721,-907,0  ],
  ["M","IND","5000",0.0029,-1440,0  ],
  ["M","IND","50M",34.2,-12.35,0  ],
  ["M","IND","55M",30.07,-13.35,0  ],
  ["M","IND","60M",23.9,-14.6,0  ],
  ["M","IND","4X200",0.312,-144,0  ],
  ["M","IND","4X400",0.0489,-340,0  ],
  ["M","IND","HJ",32.29,11.534,-5000  ],
  ["M","IND","PV",3.042,39.39,-5000  ],
  ["M","IND","LJ",1.929,48.41,-5000  ],
  ["M","IND","TJ",0.4611,98.63,-5000  ],
  ["M","IND","SP",0.042172,687.7,-20000  ],
  ["M","IND","HEP",0.00000175,53175,-5000  ],
  ["F","IND","50",33.03,-12.1,0  ],
  ["F","IND","55",27.68,-13.15,0  ],
  ["F","IND","60",24.9,-14,0  ],
  ["F","IND","200",1.962,-47.5,0  ],
  ["F","IND","300",0.6595,-79,0  ],
  ["F","IND","400",0.3224,-112,0  ],
  ["F","IND","500",0.1714,-150.5,0  ],
  ["F","IND","600",0.1063,-190.35,0  ],
  ["F","IND","800",0.0572,-264,0  ],
  ["F","IND","1000",0.03473,-340.4,0  ],
  ["F","IND","1500",0.01365,-540,0  ],
  ["F","IND","MILE",0.01154,-585.5,0  ],
  ["F","IND","2000",0.00685,-752.2,0  ],
  ["F","IND","3000",0.00259,-1200,0  ],
  ["F","IND","2M",0.002202,-1296.3,0  ],
  ["F","IND","5000",0.000825,-2100,0  ],
  ["F","IND","50M",16.2,-15.3,0  ],
  ["F","IND","55M",13.19,-16.8,0  ],
  ["F","IND","60M",11.16,-18.2,0  ],
  ["F","IND","4X200",0.0826,-212,0  ],
  ["F","IND","4X400",0.0155,-484,0  ],
  ["F","IND","HJ",39.34,10.574,-5000  ],
  ["F","IND","PV",3.953,34.83,-5000  ],
  ["F","IND","LJ",1.966,49.24,-5000  ],
  ["F","IND","TJ",0.4282,105.53,-5000  ],
  ["F","IND","SP",0.0462,657.53,-20000  ],
  ["F","IND","PEN",0.00000294,41033,-5000  ]
]